@@ -1,6 +1,6 @@
 /*
      This file is part of libmicrohttpd
-     (C) 2006-2013 Christian Grothoff (and other contributing authors)
+     (C) 2006-2014 Christian Grothoff (and other contributing authors)
 
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Lesser General Public
@@ -105,23 +105,28 @@ extern "C"
    "standard" includes won't be used (which might be a good
    idea, especially on platforms where they do not exist). */
 #ifndef MHD_PLATFORM_H
-#include <unistd.h>
 #include <stdarg.h>
 #include <stdint.h>
-#ifdef __MINGW32__
+#if defined(_WIN32) && !defined(__CYGWIN__)
 #include <ws2tcpip.h>
 #else
+#include <unistd.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #endif
 #endif
 
+#if defined(__CYGWIN__) && !defined(_SYS_TYPES_FD_SET)
+/* Do not define __USE_W32_SOCKETS under Cygwin! */
+#error Cygwin with winsock fd_set is not supported
+#endif
+
 /**
  * Current version of the library.
  * 0x01093001 = 1.9.30-1.
  */
-#define MHD_VERSION 0x00093300
+#define MHD_VERSION 0x00093400
 
 /**
  * MHD-internal return code for "YES".
@@ -156,6 +161,27 @@ extern "C"
 #define MHD_CONTENT_READER_END_WITH_ERROR (((size_t) -1LL) - 1)
 #endif
 
+#ifndef _MHD_EXTERN
+#define _MHD_EXTERN extern
+#endif
+
+#ifndef MHD_SOCKET_DEFINED
+/**
+ * MHD_socket is type for socket FDs
+ */
+#if !defined(_WIN32) || defined(_SYS_TYPES_FD_SET)
+#define MHD_POSIX_SOCKETS 1
+typedef int MHD_socket;
+#define MHD_INVALID_SOCKET (-1)
+#else /* !defined(_WIN32) || defined(_SYS_TYPES_FD_SET) */
+#define MHD_WINSOCK_SOCKETS 1
+#include <winsock2.h>
+typedef SOCKET MHD_socket;
+#define MHD_INVALID_SOCKET (INVALID_SOCKET)
+#endif /* !defined(_WIN32) || defined(_SYS_TYPES_FD_SET) */
+#define MHD_SOCKET_DEFINED 1
+#endif /* MHD_SOCKET_DEFINED */
+
 /**
  * Not all architectures and `printf()`'s support the `long long` type.
  * This gives the ability to replace `long long` with just a `long`,
@@ -499,7 +525,8 @@ enum MHD_FLAG
    * use of this option is automatic (as in, you do not even have to
    * specify it), if #MHD_USE_NO_LISTEN_SOCKET is specified.  In
    * "external" `select()` mode, this option is always simply ignored.
-   * On W32 a pair of sockets is used instead of a pipe.
+   * MHD can be build for use a pair of sockets instead of a pipe.
+   * Pair of sockets is forced on W32.
    *
    * You must also use this option if you use internal select mode
    * or a thread pool in conjunction with #MHD_add_connection.
@@ -524,7 +551,14 @@ enum MHD_FLAG
    * Enable suspend/resume functions, which also implies setting up
    * pipes to signal resume.
    */
-  MHD_USE_SUSPEND_RESUME = 8192 | MHD_USE_PIPE_FOR_SHUTDOWN
+  MHD_USE_SUSPEND_RESUME = 8192 | MHD_USE_PIPE_FOR_SHUTDOWN,
+
+  /**
+   * Enable TCP_FASTOPEN option.  This option is only available on Linux with a
+   * kernel >= 3.6.  On other systems, using this option cases #MHD_start_daemon
+   * to fail.
+   */
+  MHD_USE_TCP_FASTOPEN = 16384
 
 };
 
@@ -790,7 +824,16 @@ enum MHD_OPTION
    * to access the SNI data using `gnutls_server_name_get()`.
    * Using this option requires GnuTLS 3.0 or higher.
    */
-  MHD_OPTION_HTTPS_CERT_CALLBACK = 22
+  MHD_OPTION_HTTPS_CERT_CALLBACK = 22,
+
+  /**
+   * When using #MHD_USE_TCP_FASTOPEN, this option changes the default TCP
+   * fastopen queue length of 50.  Note that having a larger queue size can
+   * cause resource exhaustion attack as the TCP stack has to now allocate
+   * resources for the SYN packet along with its DATA.  This option should be
+   * followed by an `unsigned int` argument.
+   */
+  MHD_OPTION_TCP_FASTOPEN_QUEUE_SIZE = 23
 
 };
 
@@ -944,7 +987,7 @@ union MHD_ConnectionInfo
   /**
    * Connect socket
    */
-  int connect_fd;
+  MHD_socket connect_fd;
 
   /**
    * GNUtls session handle, of type "gnutls_session_t".
@@ -1287,7 +1330,7 @@ typedef int
  * @return NULL on error, handle to daemon on success
  * @ingroup event
  */
-struct MHD_Daemon *
+_MHD_EXTERN struct MHD_Daemon *
 MHD_start_daemon_va (unsigned int flags,
 		     uint16_t port,
 		     MHD_AcceptPolicyCallback apc, void *apc_cls,
@@ -1311,7 +1354,7 @@ MHD_start_daemon_va (unsigned int flags,
  * @return NULL on error, handle to daemon on success
  * @ingroup event
  */
-struct MHD_Daemon *
+_MHD_EXTERN struct MHD_Daemon *
 MHD_start_daemon (unsigned int flags,
 		  uint16_t port,
 		  MHD_AcceptPolicyCallback apc, void *apc_cls,
@@ -1331,14 +1374,14 @@ MHD_start_daemon (unsigned int flags,
  * Note that some thread modes require the caller to have passed
  * #MHD_USE_PIPE_FOR_SHUTDOWN when using this API.  If this daemon is
  * in one of those modes and this option was not given to
- * #MHD_start_daemon, this function will return -1.
+ * #MHD_start_daemon, this function will return #MHD_INVALID_SOCKET.
  *
  * @param daemon daemon to stop accepting new connections for
- * @return old listen socket on success, -1 if the daemon was
- *         already not listening anymore
+ * @return old listen socket on success, #MHD_INVALID_SOCKET if
+ *         the daemon was already not listening anymore
  * @ingroup specialized
  */
-int
+_MHD_EXTERN MHD_socket
 MHD_quiesce_daemon (struct MHD_Daemon *daemon);
 
 
@@ -1348,7 +1391,7 @@ MHD_quiesce_daemon (struct MHD_Daemon *daemon);
  * @param daemon daemon to stop
  * @ingroup event
  */
-void
+_MHD_EXTERN void
 MHD_stop_daemon (struct MHD_Daemon *daemon);
 
 
@@ -1381,15 +1424,18 @@ MHD_stop_daemon (struct MHD_Daemon *daemon);
  *        set to indicate further details about the error.
  * @ingroup specialized
  */
-int
+_MHD_EXTERN int
 MHD_add_connection (struct MHD_Daemon *daemon,
-		    int client_socket,
+		    MHD_socket client_socket,
 		    const struct sockaddr *addr,
 		    socklen_t addrlen);
 
 
 /**
  * Obtain the `select()` sets for this daemon.
+ * Daemon's FDs will be added to fd_sets. To get only
+ * daemon FDs in fd_sets, call FD_ZERO for each fd_set
+ * before calling this function.
  *
  * @param daemon daemon to get sets from
  * @param read_fd_set read set
@@ -1402,12 +1448,12 @@ MHD_add_connection (struct MHD_Daemon *daemon,
  *         options for this call.
  * @ingroup event
  */
-int
+_MHD_EXTERN int
 MHD_get_fdset (struct MHD_Daemon *daemon,
                fd_set *read_fd_set,
                fd_set *write_fd_set,
 	       fd_set *except_fd_set,
-	       int *max_fd);
+	       MHD_socket *max_fd);
 
 
 /**
@@ -1424,7 +1470,7 @@ MHD_get_fdset (struct MHD_Daemon *daemon,
  *        necessiate the use of a timeout right now).
  * @ingroup event
  */
-int
+_MHD_EXTERN int
 MHD_get_timeout (struct MHD_Daemon *daemon,
 		 MHD_UNSIGNED_LONG_LONG *timeout);
 
@@ -1448,7 +1494,7 @@ MHD_get_timeout (struct MHD_Daemon *daemon,
  *         options for this call.
  * @ingroup event
  */
-int
+_MHD_EXTERN int
 MHD_run (struct MHD_Daemon *daemon);
 
 
@@ -1471,7 +1517,7 @@ MHD_run (struct MHD_Daemon *daemon);
  * @return #MHD_NO on serious errors, #MHD_YES on success
  * @ingroup event
  */
-int
+_MHD_EXTERN int
 MHD_run_from_select (struct MHD_Daemon *daemon,
 		     const fd_set *read_fd_set,
 		     const fd_set *write_fd_set,
@@ -1493,7 +1539,7 @@ MHD_run_from_select (struct MHD_Daemon *daemon,
  * @return number of entries iterated over
  * @ingroup request
  */
-int
+_MHD_EXTERN int
 MHD_get_connection_values (struct MHD_Connection *connection,
                            enum MHD_ValueKind kind,
                            MHD_KeyValueIterator iterator, void *iterator_cls);
@@ -1524,7 +1570,7 @@ MHD_get_connection_values (struct MHD_Connection *connection,
  *         #MHD_YES on success
  * @ingroup request
  */
-int
+_MHD_EXTERN int
 MHD_set_connection_value (struct MHD_Connection *connection,
                           enum MHD_ValueKind kind,
                           const char *key,
@@ -1547,7 +1593,7 @@ MHD_set_connection_value (struct MHD_Connection *connection,
  * @param cls passed to @a cb
  * @ingroup logging
  */
-void
+_MHD_EXTERN void
 MHD_set_panic_func (MHD_PanicCallback cb, void *cls);
 
 
@@ -1561,7 +1607,7 @@ MHD_set_panic_func (MHD_PanicCallback cb, void *cls);
  * @return NULL if no such item was found
  * @ingroup request
  */
-const char *
+_MHD_EXTERN const char *
 MHD_lookup_connection_value (struct MHD_Connection *connection,
 			     enum MHD_ValueKind kind,
 			     const char *key);
@@ -1578,7 +1624,7 @@ MHD_lookup_connection_value (struct MHD_Connection *connection,
  *         #MHD_YES on success or if message has been queued
  * @ingroup response
  */
-int
+_MHD_EXTERN int
 MHD_queue_response (struct MHD_Connection *connection,
                     unsigned int status_code,
 		    struct MHD_Response *response);
@@ -1611,7 +1657,7 @@ MHD_queue_response (struct MHD_Connection *connection,
  *
  * @param connection the connection to suspend
  */
-void
+_MHD_EXTERN void
 MHD_suspend_connection (struct MHD_Connection *connection);
 
 
@@ -1623,7 +1669,7 @@ MHD_suspend_connection (struct MHD_Connection *connection);
  *
  * @param connection the connection to resume
  */
-void
+_MHD_EXTERN void
 MHD_resume_connection (struct MHD_Connection *connection);
 
 
@@ -1645,7 +1691,7 @@ MHD_resume_connection (struct MHD_Connection *connection);
  * @return NULL on error (i.e. invalid arguments, out of memory)
  * @ingroup response
  */
-struct MHD_Response *
+_MHD_EXTERN struct MHD_Response *
 MHD_create_response_from_callback (uint64_t size,
 				   size_t block_size,
 				   MHD_ContentReaderCallback crc, void *crc_cls,
@@ -1666,7 +1712,7 @@ MHD_create_response_from_callback (uint64_t size,
  * @deprecated use #MHD_create_response_from_buffer instead
  * @ingroup response
  */
-struct MHD_Response *
+_MHD_EXTERN struct MHD_Response *
 MHD_create_response_from_data (size_t size,
 			       void *data,
 			       int must_free,
@@ -1719,7 +1765,7 @@ enum MHD_ResponseMemoryMode
  * @return NULL on error (i.e. invalid arguments, out of memory)
  * @ingroup response
  */
-struct MHD_Response *
+_MHD_EXTERN struct MHD_Response *
 MHD_create_response_from_buffer (size_t size,
 				 void *buffer,
 				 enum MHD_ResponseMemoryMode mode);
@@ -1736,7 +1782,7 @@ MHD_create_response_from_buffer (size_t size,
  * @return NULL on error (i.e. invalid arguments, out of memory)
  * @ingroup response
  */
-struct MHD_Response *
+_MHD_EXTERN struct MHD_Response *
 MHD_create_response_from_fd (size_t size,
 			     int fd);
 
@@ -1757,7 +1803,7 @@ MHD_create_response_from_fd (size_t size,
  * @return NULL on error (i.e. invalid arguments, out of memory)
  * @ingroup response
  */
-struct MHD_Response *
+_MHD_EXTERN struct MHD_Response *
 MHD_create_response_from_fd_at_offset (size_t size,
 				       int fd,
 				       off_t offset);
@@ -1906,7 +1952,7 @@ MHD_create_response_for_upgrade (MHD_UpgradeHandler upgrade_handler,
  * @param response response to destroy
  * @ingroup response
  */
-void
+_MHD_EXTERN void
 MHD_destroy_response (struct MHD_Response *response);
 
 
@@ -1920,7 +1966,7 @@ MHD_destroy_response (struct MHD_Response *response);
  *         or out of memory
  * @ingroup response
  */
-int
+_MHD_EXTERN int
 MHD_add_response_header (struct MHD_Response *response,
                          const char *header,
 			 const char *content);
@@ -1935,7 +1981,7 @@ MHD_add_response_header (struct MHD_Response *response,
  * @return #MHD_NO on error (i.e. invalid footer or content format).
  * @ingroup response
  */
-int
+_MHD_EXTERN int
 MHD_add_response_footer (struct MHD_Response *response,
                          const char *footer,
 			 const char *content);
@@ -1950,7 +1996,7 @@ MHD_add_response_footer (struct MHD_Response *response,
  * @return #MHD_NO on error (no such header known)
  * @ingroup response
  */
-int
+_MHD_EXTERN int
 MHD_del_response_header (struct MHD_Response *response,
                          const char *header,
 			 const char *content);
@@ -1966,7 +2012,7 @@ MHD_del_response_header (struct MHD_Response *response,
  * @return number of entries iterated over
  * @ingroup response
  */
-int
+_MHD_EXTERN int
 MHD_get_response_headers (struct MHD_Response *response,
                           MHD_KeyValueIterator iterator, void *iterator_cls);
 
@@ -1979,7 +2025,7 @@ MHD_get_response_headers (struct MHD_Response *response,
  * @return NULL if header does not exist
  * @ingroup response
  */
-const char *
+_MHD_EXTERN const char *
 MHD_get_response_header (struct MHD_Response *response,
 			 const char *key);
 
@@ -2011,7 +2057,7 @@ MHD_get_response_header (struct MHD_Response *response,
  *         otherwise a PP handle
  * @ingroup request
  */
-struct MHD_PostProcessor *
+_MHD_EXTERN struct MHD_PostProcessor *
 MHD_create_post_processor (struct MHD_Connection *connection,
 			   size_t buffer_size,
 			   MHD_PostDataIterator iter, void *iter_cls);
@@ -2030,7 +2076,7 @@ MHD_create_post_processor (struct MHD_Connection *connection,
  *         (out-of-memory, iterator aborted, parse error)
  * @ingroup request
  */
-int
+_MHD_EXTERN int
 MHD_post_process (struct MHD_PostProcessor *pp,
                   const char *post_data, size_t post_data_len);
 
@@ -2045,7 +2091,7 @@ MHD_post_process (struct MHD_PostProcessor *pp,
  *                value of this function
  * @ingroup request
  */
-int
+_MHD_EXTERN int
 MHD_destroy_post_processor (struct MHD_PostProcessor *pp);
 
 
@@ -2068,7 +2114,7 @@ MHD_destroy_post_processor (struct MHD_PostProcessor *pp);
  * 			to the username if found
  * @ingroup authentication
  */
-char *
+_MHD_EXTERN char *
 MHD_digest_auth_get_username (struct MHD_Connection *connection);
 
 
@@ -2085,7 +2131,7 @@ MHD_digest_auth_get_username (struct MHD_Connection *connection);
  * 			#MHD_INVALID_NONCE if nonce is invalid
  * @ingroup authentication
  */
-int
+_MHD_EXTERN int
 MHD_digest_auth_check (struct MHD_Connection *connection,
 		       const char *realm,
 		       const char *username,
@@ -2107,7 +2153,7 @@ MHD_digest_auth_check (struct MHD_Connection *connection,
  * @return #MHD_YES on success, #MHD_NO otherwise
  * @ingroup authentication
  */
-int
+_MHD_EXTERN int
 MHD_queue_auth_fail_response (struct MHD_Connection *connection,
 			      const char *realm,
 			      const char *opaque,
@@ -2124,7 +2170,7 @@ MHD_queue_auth_fail_response (struct MHD_Connection *connection,
  * 			to the username if found
  * @ingroup authentication
  */
-char *
+_MHD_EXTERN char *
 MHD_basic_auth_get_username_password (struct MHD_Connection *connection,
 				      char** password);
 
@@ -2141,7 +2187,7 @@ MHD_basic_auth_get_username_password (struct MHD_Connection *connection,
  * @return #MHD_YES on success, #MHD_NO otherwise
  * @ingroup authentication
  */
-int
+_MHD_EXTERN int
 MHD_queue_basic_auth_fail_response (struct MHD_Connection *connection,
 				    const char *realm,
 				    struct MHD_Response *response);
@@ -2159,7 +2205,7 @@ MHD_queue_basic_auth_fail_response (struct MHD_Connection *connection,
  *         (or if the @a info_type is unknown)
  * @ingroup specialized
  */
-const union MHD_ConnectionInfo *
+_MHD_EXTERN const union MHD_ConnectionInfo *
 MHD_get_connection_info (struct MHD_Connection *connection,
 			 enum MHD_ConnectionInfoType info_type,
 			 ...);
@@ -2191,7 +2237,7 @@ enum MHD_CONNECTION_OPTION
  * @return #MHD_YES on success, #MHD_NO if setting the option failed
  * @ingroup specialized
  */
-int
+_MHD_EXTERN int
 MHD_set_connection_option (struct MHD_Connection *connection,
 			   enum MHD_CONNECTION_OPTION option,
 			   ...);
@@ -2217,7 +2263,7 @@ union MHD_DaemonInfo
   /**
    * Listen socket file descriptor
    */
-  int listen_fd;
+  MHD_socket listen_fd;
 };
 
 
@@ -2232,7 +2278,7 @@ union MHD_DaemonInfo
  *         (or if the @a info_type is unknown)
  * @ingroup specialized
  */
-const union MHD_DaemonInfo *
+_MHD_EXTERN const union MHD_DaemonInfo *
 MHD_get_daemon_info (struct MHD_Daemon *daemon,
 		     enum MHD_DaemonInfoType info_type,
 		     ...);
@@ -2244,7 +2290,7 @@ MHD_get_daemon_info (struct MHD_Daemon *daemon,
  * @return static version string, e.g. "0.9.9"
  * @ingroup specialized
  */
-const char*
+_MHD_EXTERN const char*
 MHD_get_version (void);
 
 
