@@ -29,6 +29,7 @@
 
 #include "platform.h"
 #include "microhttpd.h"
+#include "platform_interface.h"
 #if HTTPS_SUPPORT
 #include <gnutls/gnutls.h>
 #if GNUTLS_VERSION_MAJOR >= 3
@@ -38,6 +39,10 @@
 #if EPOLL_SUPPORT
 #include <sys/epoll.h>
 #endif
+#if HAVE_NETINET_TCP_H
+/* for TCP_FASTOPEN */
+#include <netinet/tcp.h>
+#endif
 
 
 /**
@@ -711,11 +716,11 @@ struct MHD_Connection
   int client_aware;
 
   /**
-   * Socket for this connection.  Set to -1 if
+   * Socket for this connection.  Set to MHD_INVALID_SOCKET if
    * this connection has died (daemon should clean
    * up in that case).
    */
-  int socket_fd;
+  MHD_socket socket_fd;
 
   /**
    * Has this socket been closed for reading (i.e.  other side closed
@@ -782,13 +787,13 @@ struct MHD_Connection
    * otherwise, this is the size of the current chunk.  A value of
    * zero is also used when we're at the end of the chunks.
    */
-  unsigned int current_chunk_size;
+  size_t current_chunk_size;
 
   /**
    * If we are receiving with chunked encoding, where are we currently
    * with respect to the current chunk (at what offset / position)?
    */
-  unsigned int current_chunk_offset;
+  size_t current_chunk_offset;
 
   /**
    * Handler used for processing read connection operations
@@ -1082,7 +1087,7 @@ struct MHD_Daemon
   /**
    * Listen socket.
    */
-  int socket_fd;
+  MHD_socket socket_fd;
 
 #if EPOLL_SUPPORT
   /**
@@ -1101,9 +1106,10 @@ struct MHD_Daemon
    * Pipe we use to signal shutdown, unless
    * 'HAVE_LISTEN_SHUTDOWN' is defined AND we have a listen
    * socket (which we can then 'shutdown' to stop listening).
-   * On W32 this is a socketpair, not a pipe.
+   * MHD can be build with usage of socketpair instead of
+   * pipe (forced on W32).
    */
-  int wpipe[2];
+  MHD_pipe wpipe[2];
 
   /**
    * Are we shutting down?
@@ -1217,7 +1223,7 @@ struct MHD_Daemon
   /**
    * Size of `digest_auth_random.
    */
-  unsigned int digest_auth_rand_size;
+  size_t digest_auth_rand_size;
 
   /**
    * Size of the nonce-nc array.
@@ -1226,6 +1232,12 @@ struct MHD_Daemon
 
 #endif
 
+#ifdef TCP_FASTOPEN
+  /**
+   * The queue size for incoming SYN + DATA packets.
+   */
+  unsigned int fastopen_queue_size;
+#endif
 };
 
 
