@@ -27,14 +27,18 @@
 
 #include "MHD_config.h"
 #include "platform.h"
+#include "platform_interface.h"
 #include <curl/curl.h>
 #include <microhttpd.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
 
-#ifdef __MINGW32__
-#define usleep(usec) (Sleep ((usec) / 1000),0)
+#ifdef _WIN32
+#ifndef WIN32_LEAN_AND_MEAN
+#define WIN32_LEAN_AND_MEAN 1
+#endif /* !WIN32_LEAN_AND_MEAN */
+#include <windows.h>
 #endif
 
 #ifndef WINDOWS
@@ -259,7 +263,7 @@ testExternalGet ()
   fd_set rs;
   fd_set ws;
   fd_set es;
-  int max;
+  MHD_socket max;
   int running; 
   struct CURLMsg *msg;
   time_t start;
@@ -448,7 +452,7 @@ static int
 testStopRace (int poll_flag)
 {
     struct sockaddr_in sin;
-    int fd;
+    MHD_socket fd;
     struct MHD_Daemon *d;
     
     d = MHD_start_daemon (MHD_USE_THREAD_PER_CONNECTION | MHD_USE_DEBUG | poll_flag,
@@ -457,10 +461,10 @@ testStopRace (int poll_flag)
     if (d == NULL)
        return 16;
     
-    fd = SOCKET (PF_INET, SOCK_STREAM, 0);
-    if (fd < 0)
+    fd = socket (PF_INET, SOCK_STREAM, 0);
+    if (fd == MHD_INVALID_SOCKET)
     {
-       fprintf(stderr, "socket: %m\n");
+       fprintf(stderr, "socket error\n");
        return 256;
     }
     
@@ -469,10 +473,10 @@ testStopRace (int poll_flag)
     sin.sin_port = htons(1081);
     sin.sin_addr.s_addr = htonl(0x7f000001);
     
-    if (CONNECT (fd, (struct sockaddr *)(&sin), sizeof(sin)) < 0)
+    if (connect (fd, (struct sockaddr *)(&sin), sizeof(sin)) < 0)
     {
-       fprintf(stderr, "connect: %m\n");
-       CLOSE (fd);
+       fprintf(stderr, "connect error\n");
+       MHD_socket_close_ (fd);
        return 512;
     }
     
@@ -483,7 +487,7 @@ testStopRace (int poll_flag)
     /* printf("Stopping daemon\n"); */
     MHD_stop_daemon (d);
  
-    CLOSE (fd);
+    MHD_socket_close_ (fd);
     
     /* printf("good\n"); */
     return 0;
