@@ -26,6 +26,7 @@
 
 #include "MHD_config.h"
 #include "platform.h"
+#include "platform_interface.h"
 #include <curl/curl.h>
 #include <microhttpd.h>
 #include <stdlib.h>
@@ -93,7 +94,7 @@ ahc_echo (void *cls,
     {
       fprintf (stderr, "Failed to open `%s': %s\n",
 	       sourcefile,
-	       STRERROR (errno));
+	       MHD_strerror_ (errno));
       exit (1);
     }
   response = MHD_create_response_from_fd (strlen (TESTSTR), fd);
@@ -265,7 +266,7 @@ testExternalGet ()
   fd_set rs;
   fd_set ws;
   fd_set es;
-  int max;
+  MHD_socket max;
   int running;
   struct CURLMsg *msg;
   time_t start;
@@ -457,13 +458,13 @@ main (int argc, char *const *argv)
   FILE *f;
 
   if ( (NULL == (tmp = getenv ("TMPDIR"))) &&
-       (NULL == (tmp = getenv ("TMP"))) )
+       (NULL == (tmp = getenv ("TMP"))) &&
+       (NULL == (tmp = getenv ("TEMP"))) )
     tmp = "/tmp";
   sourcefile = malloc (strlen (tmp) + 32);
   sprintf (sourcefile,
-	   "%s%s%s",
+	   "%s/%s",
 	   tmp,
-	   DIR_SEPARATOR_STR,
 	   "test-mhd-sendfile");
   f = fopen (sourcefile, "w");
   if (NULL == f)
@@ -485,7 +486,7 @@ main (int argc, char *const *argv)
   if (errorCount != 0)
     fprintf (stderr, "Error (code: %u)\n", errorCount);
   curl_global_cleanup ();
-  UNLINK (sourcefile);
+  unlink (sourcefile);
   free (sourcefile);
   return errorCount != 0;       /* 0 == pass */
 }
