@@ -31,7 +31,9 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#ifdef HAVE_GCRYPT_H
 #include <gcrypt.h>
+#endif
 
 #ifndef WINDOWS
 #include <unistd.h>
@@ -342,7 +344,7 @@ testExternalPost ()
   fd_set rs;
   fd_set ws;
   fd_set es;
-  int max;
+  MHD_socket max;
   int running;
   struct CURLMsg *msg;
   time_t start;
@@ -468,9 +470,11 @@ main (int argc, char *const *argv)
 {
   unsigned int errorCount = 0;
 
+#ifdef HAVE_GCRYPT_H
   gcry_control (GCRYCTL_ENABLE_QUICK_RANDOM, 0);
 #ifdef GCRYCTL_INITIALIZATION_FINISHED
   gcry_control (GCRYCTL_INITIALIZATION_FINISHED, 0);
+#endif
 #endif
   oneone = NULL != strstr (argv[0], "11");
   if (0 != curl_global_init (CURL_GLOBAL_WIN32))
